@@ -3,21 +3,23 @@ Base class for GeoJSON services.
 
 Fetches GeoJSON feed from URL to be defined by sub-class.
 """
-from datetime import datetime
-import geojson
 import logging
-
-import requests
-from geojson import Point, GeometryCollection, Polygon
-from haversine import haversine
+from datetime import datetime
 from json import JSONDecodeError
-from typing import Optional
+from typing import Callable, Dict, List, Optional, Tuple
+
+import geojson
+import requests
+from geojson import GeometryCollection, Point, Polygon
+from haversine import haversine
+
+from geojson_client.consts import FILTER_RADIUS, HTTP_ACCEPT_ENCODING_HEADER
 
 _LOGGER = logging.getLogger(__name__)
 
-UPDATE_OK = 'OK'
-UPDATE_OK_NO_DATA = 'OK_NO_DATA'
-UPDATE_ERROR = 'ERROR'
+UPDATE_OK = "OK"
+UPDATE_OK_NO_DATA = "OK_NO_DATA"
+UPDATE_ERROR = "ERROR"
 
 
 class GeoJsonFeed:
@@ -28,20 +30,27 @@ class GeoJsonFeed:
         self._home_coordinates = home_coordinates
         self._filter_radius = filter_radius
         self._url = url
-        self._request = requests.Request(method="GET", url=url).prepare()
+        self._request = requests.Request(
+            method="GET", url=url, headers=HTTP_ACCEPT_ENCODING_HEADER
+        ).prepare()
         self._last_timestamp = None
 
     def __repr__(self):
         """Return string representation of this feed."""
-        return '<{}(home={}, url={}, radius={})>'.format(
-            self.__class__.__name__, self._home_coordinates, self._url,
-            self._filter_radius)
+        return "<{}(home={}, url={}, radius={})>".format(
+            self.__class__.__name__,
+            self._home_coordinates,
+            self._url,
+            self._filter_radius,
+        )
 
     def _new_entry(self, home_coordinates, feature, global_data):
         """Generate a new entry."""
         pass
 
-    def update(self):
+    def _update_internal(
+        self, filter_function: Callable[[List], List]
+    ) -> Tuple[str, Optional[List]]:
         """Update from external source and return filtered entries."""
         status, data = self._fetch()
         if status == UPDATE_OK:
@@ -50,11 +59,11 @@ class GeoJsonFeed:
                 global_data = self._extract_from_feed(data)
                 # Extract data from feed entries.
                 for feature in data.features:
-                    entries.append(self._new_entry(self._home_coordinates,
-                                                   feature, global_data))
-                filtered_entries = self._filter_entries(entries)
-                self._last_timestamp = self._extract_last_timestamp(
-                    filtered_entries)
+                    entries.append(
+                        self._new_entry(self._home_coordinates, feature, global_data)
+                    )
+                filtered_entries = filter_function(entries)
+                self._last_timestamp = self._extract_last_timestamp(filtered_entries)
                 return UPDATE_OK, filtered_entries
             else:
                 # Should not happen.
@@ -66,6 +75,21 @@ class GeoJsonFeed:
             # Error happened while fetching the feed.
             return UPDATE_ERROR, None
 
+    def update(self) -> Tuple[str, Optional[List]]:
+        """Update from external source and return filtered entries."""
+        return self._update_internal(lambda entries: self._filter_entries(entries))
+
+    def update_override(
+        self, filter_overrides: Dict = None
+    ) -> Tuple[str, Optional[List]]:
+        """Update from external source and return filtered entries with ability to
+        override filter conditions."""
+        return self._update_internal(
+            lambda entries: self._filter_entries_override(
+                entries, filter_overrides=filter_overrides
+            )
+        )
+
     def _fetch(self):
         """Fetch GeoJSON data from external source."""
         try:
@@ -77,31 +101,45 @@ class GeoJsonFeed:
             else:
                 _LOGGER.warning(
                     "Fetching data from %s failed with status %s",
-                    self._request.url, response.status_code)
+                    self._request.url,
+                    response.status_code,
+                )
                 return UPDATE_ERROR, None
         except requests.exceptions.RequestException as request_ex:
-            _LOGGER.warning("Fetching data from %s failed with %s",
-                            self._request.url, request_ex)
+            _LOGGER.warning(
+                "Fetching data from %s failed with %s", self._request.url, request_ex
+            )
             return UPDATE_ERROR, None
         except JSONDecodeError as decode_ex:
-            _LOGGER.warning("Unable to parse JSON from %s: %s",
-                            self._request.url, decode_ex)
+            _LOGGER.warning(
+                "Unable to parse JSON from %s: %s", self._request.url, decode_ex
+            )
             return UPDATE_ERROR, None
 
     def _filter_entries(self, entries):
+        """Filter the provided entries."""
+        return self._filter_entries_override(entries, None)
+
+    def _filter_entries_override(self, entries, filter_overrides: Dict = None):
         """Filter the provided entries."""
         filtered_entries = entries
         # Always remove entries without geometry
         filtered_entries = list(
-            filter(lambda entry:
-                   entry.geometry is not None,
-                   filtered_entries))
+            filter(lambda entry: entry.geometry is not None, filtered_entries)
+        )
         # Filter by distance.
-        if self._filter_radius:
+        filter_radius = (
+            filter_overrides[FILTER_RADIUS]
+            if filter_overrides and FILTER_RADIUS in filter_overrides
+            else self._filter_radius
+        )
+        if filter_radius:
             filtered_entries = list(
-                filter(lambda entry:
-                       entry.distance_to_home <= self._filter_radius,
-                       filtered_entries))
+                filter(
+                    lambda entry: entry.distance_to_home <= filter_radius,
+                    filtered_entries,
+                )
+            )
         return filtered_entries
 
     def _extract_from_feed(self, feed):
@@ -128,7 +166,7 @@ class FeedEntry:
 
     def __repr__(self):
         """Return string representation of this entry."""
-        return '<{}(id={})>'.format(self.__class__.__name__, self.external_id)
+        return "<{}(id={})>".format(self.__class__.__name__, self.external_id)
 
     @property
     def geometry(self):
@@ -163,7 +201,8 @@ class FeedEntry:
     def distance_to_home(self):
         """Return the distance in km of this entry to the home coordinates."""
         return GeoJsonDistanceHelper.distance_to_geometry(
-            self._home_coordinates, self.geometry)
+            self._home_coordinates, self.geometry
+        )
 
     def _search_in_feature(self, name):
         """Find an attribute in the feature object."""
@@ -173,8 +212,11 @@ class FeedEntry:
 
     def _search_in_properties(self, name):
         """Find an attribute in the feed entry's properties."""
-        if self._feature and self._feature.properties \
-                and name in self._feature.properties:
+        if (
+            self._feature
+            and self._feature.properties
+            and name in self._feature.properties
+        ):
             return self._feature.properties[name]
         return None
 
@@ -192,14 +234,12 @@ class GeoJsonDistanceHelper:
         latitude = longitude = None
         if isinstance(geometry, Point):
             # Just extract latitude and longitude directly.
-            latitude, longitude = geometry.coordinates[1], \
-                                  geometry.coordinates[0]
+            latitude, longitude = geometry.coordinates[1], geometry.coordinates[0]
         elif isinstance(geometry, GeometryCollection):
             # Go through the collection, and extract the first suitable
             # geometry.
             for entry in geometry.geometries:
-                latitude, longitude = \
-                    GeoJsonDistanceHelper.extract_coordinates(entry)
+                latitude, longitude = GeoJsonDistanceHelper.extract_coordinates(entry)
                 if latitude is not None and longitude is not None:
                     break
         elif isinstance(geometry, Polygon):
@@ -209,8 +249,9 @@ class GeoJsonDistanceHelper:
             number_of_points = len(geometry.coordinates[0])
             longitude = sum(longitudes_list) / number_of_points
             latitude = sum(latitudes_list) / number_of_points
-            _LOGGER.debug("Centroid of %s is %s", geometry.coordinates[0],
-                          (latitude, longitude))
+            _LOGGER.debug(
+                "Centroid of %s is %s", geometry.coordinates[0], (latitude, longitude)
+            )
         else:
             _LOGGER.debug("Not implemented: %s", type(geometry))
         return latitude, longitude
@@ -221,13 +262,16 @@ class GeoJsonDistanceHelper:
         distance = float("inf")
         if isinstance(geometry, Point):
             distance = GeoJsonDistanceHelper._distance_to_point(
-                home_coordinates, geometry)
+                home_coordinates, geometry
+            )
         elif isinstance(geometry, GeometryCollection):
             distance = GeoJsonDistanceHelper._distance_to_geometry_collection(
-                home_coordinates, geometry)
+                home_coordinates, geometry
+            )
         elif isinstance(geometry, Polygon):
             distance = GeoJsonDistanceHelper._distance_to_polygon(
-                home_coordinates, geometry.coordinates[0])
+                home_coordinates, geometry.coordinates[0]
+            )
         else:
             _LOGGER.debug("Not implemented: %s", type(geometry))
         return distance
@@ -237,18 +281,19 @@ class GeoJsonDistanceHelper:
         """Calculate the distance between home coordinates and the point."""
         # Swap coordinates to match: (latitude, longitude).
         return GeoJsonDistanceHelper._distance_to_coordinates(
-            home_coordinates, (point.coordinates[1], point.coordinates[0]))
+            home_coordinates, (point.coordinates[1], point.coordinates[0])
+        )
 
     @staticmethod
-    def _distance_to_geometry_collection(home_coordinates,
-                                         geometry_collection):
+    def _distance_to_geometry_collection(home_coordinates, geometry_collection):
         """Calculate the distance between home coordinates and the geometry
         collection."""
         distance = float("inf")
         for geometry in geometry_collection.geometries:
-            distance = min(distance,
-                           GeoJsonDistanceHelper.distance_to_geometry(
-                               home_coordinates, geometry))
+            distance = min(
+                distance,
+                GeoJsonDistanceHelper.distance_to_geometry(home_coordinates, geometry),
+            )
         return distance
 
     @staticmethod
@@ -259,10 +304,12 @@ class GeoJsonDistanceHelper:
         # to each point of the polygon but not to each edge of the
         # polygon; should be good enough
         for polygon_point in polygon:
-            distance = min(distance,
-                           GeoJsonDistanceHelper._distance_to_coordinates(
-                               home_coordinates,
-                               (polygon_point[1], polygon_point[0])))
+            distance = min(
+                distance,
+                GeoJsonDistanceHelper._distance_to_coordinates(
+                    home_coordinates, (polygon_point[1], polygon_point[0])
+                ),
+            )
         return distance
 
     @staticmethod
